@@ -1,10 +1,11 @@
 /* 
  * GDA common library
- * Copyright (C) 1998-2002 The GNOME Foundation.
+ * Copyright (C) 1998 - 2004 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
  *	Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -26,26 +27,16 @@
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-util.h>
-#include <libgda/gda-xml-database.h>
+#include <libgda/gda-row.h>
 #include <string.h>
 
 #define PARENT_TYPE G_TYPE_OBJECT
 #define CLASS(model) (GDA_DATA_MODEL_CLASS (G_OBJECT_GET_CLASS (model)))
 
-struct _GdaDataModelPrivate {
-	gboolean notify_changes;
-	GHashTable *column_titles;
-	gchar *cmd_text;
-	GdaCommandType cmd_type;
 
-	/* update mode */
-	gboolean updating;
-};
-
-static void gda_data_model_class_init (GdaDataModelClass *klass);
-static void gda_data_model_init       (GdaDataModel *model, GdaDataModelClass *klass);
-static void gda_data_model_finalize   (GObject *object);
+static void gda_data_model_iface_init (gpointer g_class);
 
+/* signals */
 enum {
 	CHANGED,
 	ROW_INSERTED,
@@ -56,165 +47,11 @@ enum {
 	COLUMN_REMOVED,
 	BEGIN_UPDATE,
 	CANCEL_UPDATE,
-	END_UPDATE,
+	COMMIT_UPDATE,
 	LAST_SIGNAL
 };
 
 static guint gda_data_model_signals[LAST_SIGNAL];
-static GObjectClass *parent_class = NULL;
-
-/*
- * GdaDataModel class implementation
- */
-
-static void
-gda_data_model_class_init (GdaDataModelClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	parent_class = g_type_class_peek_parent (klass);
-
-	gda_data_model_signals[CHANGED] =
-		g_signal_new ("changed",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, changed),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE, 0);
-	gda_data_model_signals[ROW_INSERTED] =
-		g_signal_new ("row_inserted",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, row_inserted),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE, 1, G_TYPE_INT);
-	gda_data_model_signals[ROW_UPDATED] =
-		g_signal_new ("row_updated",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, row_updated),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE, 1, G_TYPE_INT);
-	gda_data_model_signals[ROW_REMOVED] =
-		g_signal_new ("row_removed",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, row_removed),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE, 1, G_TYPE_INT);
-	gda_data_model_signals[COLUMN_INSERTED] =
-		g_signal_new ("column_inserted",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, column_inserted),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE, 1, G_TYPE_INT);
-	gda_data_model_signals[COLUMN_UPDATED] =
-		g_signal_new ("column_updated",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, column_updated),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE, 1, G_TYPE_INT);
-	gda_data_model_signals[COLUMN_REMOVED] =
-		g_signal_new ("column_removed",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, column_removed),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE, 1, G_TYPE_INT);
-	gda_data_model_signals[BEGIN_UPDATE] =
-		g_signal_new ("begin_update",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, begin_update),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE, 0);
-	gda_data_model_signals[CANCEL_UPDATE] =
-		g_signal_new ("cancel_update",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, cancel_update),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE, 0);
-	gda_data_model_signals[END_UPDATE] =
-		g_signal_new ("end_update",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdaDataModelClass, end_update),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE, 0);
-
-	object_class->finalize = gda_data_model_finalize;
-	klass->changed = NULL;
-	klass->begin_update = NULL;
-	klass->cancel_update = NULL;
-	klass->end_update = NULL;
-	klass->get_n_rows = NULL;
-	klass->get_n_columns = NULL;
-	klass->describe_column = NULL;
-	klass->get_row = NULL;
-	klass->get_value_at = NULL;
-	klass->is_updatable = NULL;
-	klass->append_row = NULL;
-	klass->remove_row = NULL;
-	klass->update_row = NULL;
-	klass->append_column = NULL;
-	klass->remove_column = NULL;
-	klass->update_column = NULL;
-}
-
-static void
-gda_data_model_init (GdaDataModel *model, GdaDataModelClass *klass)
-{
-	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-
-	model->priv = g_new (GdaDataModelPrivate, 1);
-	model->priv->notify_changes = TRUE;
-	model->priv->column_titles = g_hash_table_new (g_direct_hash,
-						       g_direct_equal);
-	model->priv->updating = FALSE;
-	model->priv->cmd_text = NULL;
-	model->priv->cmd_type = GDA_COMMAND_TYPE_INVALID;
-}
-
-static void
-free_hash_string (gpointer key, gpointer value, gpointer user_data)
-{
-	g_free (value);
-}
-
-static void
-gda_data_model_finalize (GObject *object)
-{
-	GdaDataModel *model = (GdaDataModel *) object;
-
-	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-
-	/* free memory */
-	g_hash_table_foreach (model->priv->column_titles, free_hash_string, NULL);
-	g_hash_table_destroy (model->priv->column_titles);
-	model->priv->column_titles = NULL;
-
-	g_free (model->priv->cmd_text);
-	model->priv->cmd_text = NULL;
-
-	g_free (model->priv);
-	model->priv = NULL;
-
-	/* chain to parent class */
-	parent_class->finalize (object);
-}
 
 GType
 gda_data_model_get_type (void)
@@ -222,24 +59,124 @@ gda_data_model_get_type (void)
 	static GType type = 0;
 
 	if (!type) {
-		if (type == 0) {
-			static GTypeInfo info = {
-				sizeof (GdaDataModelClass),
-				(GBaseInitFunc) NULL,
-				(GBaseFinalizeFunc) NULL,
-				(GClassInitFunc) gda_data_model_class_init,
-				NULL, NULL,
-				sizeof (GdaDataModel),
-				0,
-				(GInstanceInitFunc) gda_data_model_init
-			};
-			type = g_type_register_static (PARENT_TYPE, "GdaDataModel", &info, G_TYPE_FLAG_ABSTRACT);
-		}
+		static const GTypeInfo info = {
+			sizeof (GdaDataModelIface),
+			(GBaseInitFunc) gda_data_model_iface_init,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) NULL,
+			NULL,
+			NULL,
+			0,
+			0,
+			(GInstanceInitFunc) NULL
+		};
+		
+		type = g_type_register_static (G_TYPE_INTERFACE, "GdaDataModel", &info, 0);
+		g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
 	}
-
 	return type;
 }
 
+static void
+gda_data_model_iface_init (gpointer g_class)
+{
+	static gboolean initialized = FALSE;
+
+	if (! initialized) {
+		gda_data_model_signals[CHANGED] =
+			g_signal_new ("changed",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, changed),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__VOID,
+				      G_TYPE_NONE, 0);
+		gda_data_model_signals[ROW_INSERTED] =
+			g_signal_new ("row_inserted",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, row_inserted),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__INT,
+				      G_TYPE_NONE, 1, G_TYPE_INT);
+		gda_data_model_signals[ROW_UPDATED] =
+			g_signal_new ("row_updated",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, row_updated),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__INT,
+				      G_TYPE_NONE, 1, G_TYPE_INT);
+		gda_data_model_signals[ROW_REMOVED] =
+			g_signal_new ("row_removed",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, row_removed),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__INT,
+				      G_TYPE_NONE, 1, G_TYPE_INT);
+		gda_data_model_signals[COLUMN_INSERTED] =
+			g_signal_new ("column_inserted",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, column_inserted),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__INT,
+				      G_TYPE_NONE, 1, G_TYPE_INT);
+		gda_data_model_signals[COLUMN_UPDATED] =
+			g_signal_new ("column_updated",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, column_updated),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__INT,
+				      G_TYPE_NONE, 1, G_TYPE_INT);
+		gda_data_model_signals[COLUMN_REMOVED] =
+			g_signal_new ("column_removed",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, column_removed),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__INT,
+				      G_TYPE_NONE, 1, G_TYPE_INT);
+		gda_data_model_signals[BEGIN_UPDATE] =
+			g_signal_new ("begin_update",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, begin_update),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__VOID,
+				      G_TYPE_NONE, 0);
+		gda_data_model_signals[CANCEL_UPDATE] =
+			g_signal_new ("cancel_update",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, cancel_update),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__VOID,
+				      G_TYPE_NONE, 0);
+		gda_data_model_signals[COMMIT_UPDATE] =
+			g_signal_new ("commit_update",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, commit_update),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__VOID,
+				      G_TYPE_NONE, 0);
+
+		initialized = TRUE;
+	}
+}
+
+static gboolean
+do_notify_changes (GdaDataModel *model)
+{
+	gboolean notify_changes = TRUE;
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_notify)
+		notify_changes = (GDA_DATA_MODEL_GET_IFACE (model)->i_get_notify) (model);
+	return notify_changes;
+}
+
 /**
  * gda_data_model_changed
  * @model: a #GdaDataModel object.
@@ -254,12 +191,11 @@ void
 gda_data_model_changed (GdaDataModel *model)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-
-	if (model->priv->notify_changes) {
+	
+	if (do_notify_changes (model)) 
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[CHANGED],
 			       0);
-	}
 }
 
 /**
@@ -274,7 +210,7 @@ gda_data_model_row_inserted (GdaDataModel *model, gint row)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	if (model->priv->notify_changes) {
+	if (do_notify_changes (model)) {
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[ROW_INSERTED],
 			       0, row);
@@ -295,7 +231,7 @@ gda_data_model_row_updated (GdaDataModel *model, gint row)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	if (model->priv->notify_changes) {
+	if (do_notify_changes (model)) {
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[ROW_UPDATED],
 			       0, row);
@@ -316,7 +252,7 @@ gda_data_model_row_removed (GdaDataModel *model, gint row)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	if (model->priv->notify_changes) {
+	if (do_notify_changes (model)) {
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[ROW_REMOVED],
 			       0, row);
@@ -337,7 +273,7 @@ gda_data_model_column_inserted (GdaDataModel *model, gint col)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	if (model->priv->notify_changes) {
+	if (do_notify_changes (model)) {
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[COLUMN_INSERTED],
 			       0, col);
@@ -358,7 +294,7 @@ gda_data_model_column_updated (GdaDataModel *model, gint col)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	if (model->priv->notify_changes) {
+	if (do_notify_changes (model)) {
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[COLUMN_UPDATED],
 			       0, col);
@@ -379,7 +315,7 @@ gda_data_model_column_removed (GdaDataModel *model, gint col)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	if (model->priv->notify_changes) {
+	if (do_notify_changes (model)) {
 		g_signal_emit (G_OBJECT (model),
 			       gda_data_model_signals[COLUMN_REMOVED],
 			       0, col);
@@ -400,7 +336,11 @@ void
 gda_data_model_freeze (GdaDataModel *model)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-	model->priv->notify_changes = FALSE;
+	
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_set_notify)
+		(GDA_DATA_MODEL_GET_IFACE (model)->i_set_notify) (model, FALSE);
+	else
+		g_warning ("%s() method not supported\n", __FUNCTION__);
 }
 
 /**
@@ -413,7 +353,11 @@ void
 gda_data_model_thaw (GdaDataModel *model)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-	model->priv->notify_changes = TRUE;
+
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_set_notify)
+		(GDA_DATA_MODEL_GET_IFACE (model)->i_set_notify) (model, TRUE);
+	else
+		g_warning ("%s() method not supported\n", __FUNCTION__);
 }
 
 /**
@@ -426,9 +370,13 @@ gint
 gda_data_model_get_n_rows (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), -1);
-	g_return_val_if_fail (CLASS (model)->get_n_rows != NULL, -1);
 
-	return CLASS (model)->get_n_rows (model);
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_n_rows)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_get_n_rows) (model);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return -1;
+	}
 }
 
 /**
@@ -441,9 +389,13 @@ gint
 gda_data_model_get_n_columns (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), -1);
-	g_return_val_if_fail (CLASS (model)->get_n_columns != NULL, -1);
-
-	return CLASS (model)->get_n_columns (model);
+	
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_n_columns)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_get_n_columns) (model);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return -1;
+	}
 }
 
 /**
@@ -453,39 +405,22 @@ gda_data_model_get_n_columns (GdaDataModel *model)
  *
  * Queries the underlying data model implementation for a description
  * of a given column. That description is returned in the form of
- * a #GdaFieldAttributes structure, which contains all the information
+ * a #GdaDataModelColumnAttributes structure, which contains all the information
  * about the given column in the data model.
  *
  * Returns: the description of the column.
  */
-GdaFieldAttributes *
+GdaDataModelColumnAttributes *
 gda_data_model_describe_column (GdaDataModel *model, gint col)
 {
-	GdaFieldAttributes *fa;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-	g_return_val_if_fail (CLASS (model)->describe_column != NULL, NULL);
 
-	fa = CLASS (model)->describe_column (model, col);
-	if (!fa) {
-		const GdaValue *value;
-
-		/* we generate a basic FieldAttributes structure */
-		fa = gda_field_attributes_new ();
-		gda_field_attributes_set_defined_size (fa, 0);
-		gda_field_attributes_set_name (fa, g_hash_table_lookup (model->priv->column_titles,
-									GINT_TO_POINTER (col)));
-		gda_field_attributes_set_scale (fa, 0);
-		value = gda_data_model_get_value_at (model, col, 0);
-		if (value == NULL)
-			gda_field_attributes_set_gdatype (fa, GDA_VALUE_TYPE_STRING);
-		else
-			gda_field_attributes_set_gdatype (fa, gda_value_get_type (value));
-
-		gda_field_attributes_set_allow_null (fa, TRUE);
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_describe_column)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_describe_column) (model, col);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return NULL;
 	}
-
-	return fa;
 }
 
 /**
@@ -498,34 +433,14 @@ gda_data_model_describe_column (GdaDataModel *model, gint col)
 const gchar *
 gda_data_model_get_column_title (GdaDataModel *model, gint col)
 {
-	gint n_cols;
-	gchar *title;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	n_cols = gda_data_model_get_n_columns (model);
-	if (col < n_cols && col >= 0){
-		title = g_hash_table_lookup (model->priv->column_titles,
-					     GINT_TO_POINTER (col));
-		if (title == NULL) {
-			GdaFieldAttributes *fa;
-
-			fa = gda_data_model_describe_column (model, col);
-			if (fa) {
-				gda_data_model_set_column_title (model, col, title);
-				gda_field_attributes_free (fa);
-
-				return g_hash_table_lookup (model->priv->column_titles,
-							    GINT_TO_POINTER (col));
-			}
-			else
-				title = "";
-		}
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_column_title)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_get_column_title) (model, col);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return NULL;
 	}
-	else
-		title = "";
-
-	return (const gchar *) title;
 }
 
 /**
@@ -539,25 +454,12 @@ gda_data_model_get_column_title (GdaDataModel *model, gint col)
 void
 gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *title)
 {
-	gint n_cols;
-
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 
-	n_cols = gda_data_model_get_n_columns (model);
-	if (col >= 0 && col < n_cols) {
-		gpointer key, value;
-
-		if (g_hash_table_lookup_extended (model->priv->column_titles,
-						  GINT_TO_POINTER (col),
-						  &key, &value)) {
-			g_hash_table_remove (model->priv->column_titles,
-					     GINT_TO_POINTER (col));
-			g_free (value);
-		}
-
-		g_hash_table_insert (model->priv->column_titles, 
-				     GINT_TO_POINTER (col), g_strdup (title));
-	}
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_set_column_title)
+		(GDA_DATA_MODEL_GET_IFACE (model)->i_set_column_title) (model, col, title);
+	else 
+		g_warning ("%s() method not supported\n", __FUNCTION__);
 }
 
 /**
@@ -574,23 +476,14 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
 gint
 gda_data_model_get_column_position (GdaDataModel *model, const gchar *title)
 {
-	gint n_cols;
-	gint i;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), -1);
-	g_return_val_if_fail (title != NULL, -1);
 
-	n_cols = gda_data_model_get_n_columns (model);
-	for (i = 0; i < n_cols; i++) {
-		gpointer value;
-
-		value = g_hash_table_lookup (model->priv->column_titles,
-					     GINT_TO_POINTER (i));
-		if (value && !strcmp (title, (const char *) value))
-			return i;
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_column_pos)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_get_column_pos) (model, title);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return -1;
 	}
-
-	return -1;
 }
 
 /**
@@ -606,9 +499,13 @@ const GdaRow *
 gda_data_model_get_row (GdaDataModel *model, gint row)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-	g_return_val_if_fail (CLASS (model)->get_row != NULL, NULL);
 
-	return CLASS (model)->get_row (model, row);
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_row)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_get_row) (model, row);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return NULL;
+	}
 }
 
 /**
@@ -629,9 +526,13 @@ const GdaValue *
 gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-	g_return_val_if_fail (CLASS (model)->get_value_at != NULL, NULL);
 
-	return CLASS (model)->get_value_at (model, col, row);
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_value_at)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_get_value_at) (model, col, row);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return NULL;
+	}
 }
 
 /**
@@ -646,13 +547,15 @@ gboolean
 gda_data_model_is_updatable (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (CLASS (model)->is_updatable != NULL, FALSE);
 
-	return CLASS (model)->is_updatable (model);
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_is_updatable)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_is_updatable) (model);
+	else 
+		return FALSE;
 }
 
 /**
- * gda_data_model_append_row
+ * gda_data_model_append_values
  * @model: a #GdaDataModel object.
  * @values: #GList of #GdaValue* representing the row to add.  The
  *          length must match model's column count.  These #GdaValue
@@ -660,19 +563,54 @@ gda_data_model_is_updatable (GdaDataModel *model)
  *
  * Appends a row to the given data model.
  *
+ * Deprecated: will be removed in future versions, use gda_data_model_append_row() instead.
+ *
  * Returns: the added row.
  */
 const GdaRow *
-gda_data_model_append_row (GdaDataModel *model, const GList *values)
+gda_data_model_append_values (GdaDataModel *model, const GList *values)
 {
-	const GdaRow *row;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-	g_return_val_if_fail (CLASS (model)->append_row != NULL, NULL);
 
-	row = CLASS (model)->append_row (model, values);
-	gda_data_model_row_inserted (model, gda_row_get_number ((GdaRow *) row));
-	return row;
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_append_values) {
+		const GdaRow *row;
+		row = (GDA_DATA_MODEL_GET_IFACE (model)->i_append_values) (model, values);
+		gda_data_model_row_inserted (model, gda_row_get_number ((GdaRow *) row));
+		return row;
+	}
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return NULL;
+	}
+}
+
+/**
+ * gda_data_model_append_row
+ * @model: a #GdaDataModel object.
+ * @row: the #GdaRow to be appended
+ * 
+ * Appends a row to the data model. The 'number' attribute of @row may be
+ * modified by the data model to represent the actual row position.
+ *
+ * This function differs from the gda_data_model_append_values() method in that
+ * it is more coherent with the other gda_data_model_*_row() functions, and also
+ * is more adapted to #GdaDataModel which don't manage themselves any #GdaRow internally.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
+ */
+gboolean
+gda_data_model_append_row (GdaDataModel *model, GdaRow *row)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (row != NULL, FALSE);
+
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_append_row) {
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_append_row) (model, row);
+	}
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
+	}
 }
 
 /**
@@ -688,18 +626,16 @@ gda_data_model_append_row (GdaDataModel *model, const GList *values)
 gboolean
 gda_data_model_remove_row (GdaDataModel *model, const GdaRow *row)
 {
-	gboolean result;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 	g_return_val_if_fail (row != NULL, FALSE);
-	g_return_val_if_fail (CLASS (model)->remove_row != NULL, FALSE);
 
-	result = CLASS (model)->remove_row (model, row);
-	if (result) {
-		gda_data_model_row_removed (model, gda_row_get_number ((GdaRow *) row));
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_remove_row) {
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_remove_row) (model, row);
+	}
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
 	}
-
-	return result;
 }
 
 /**
@@ -719,42 +655,39 @@ gda_data_model_update_row (GdaDataModel *model, const GdaRow *row)
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 	g_return_val_if_fail (row != NULL, FALSE);
-	g_return_val_if_fail (CLASS (model)->update_row != NULL, FALSE);
 
-	result = CLASS (model)->update_row (model, row);
-	if (result) {
-		gda_data_model_row_updated (model, gda_row_get_number ((GdaRow *) row));
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_update_row) {
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_update_row) (model, row);
+	}
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
 	}
-	return result;
 }
 
 /**
  * gda_data_model_append_column
  * @model: a #GdaDataModel object.
- * @col: a #GdaFieldAttributes describing the column to add.
+ * @attrs: a #GdaDataModelColumnAttributes describing the column to add.
  *
  * Appends a column to the given data model.  If successful, the position of
  * the new column in the data model is set on @col, and you can grab it using
- * @gda_field_attributes_get_position.
+ * @gda_data_model_column_attributes_get_position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
-gda_data_model_append_column (GdaDataModel *model, GdaFieldAttributes *col)
+gda_data_model_append_column (GdaDataModel *model, const GdaDataModelColumnAttributes *attrs)
 {
-	gboolean result;
-	
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (CLASS (model)->append_column != NULL, FALSE);
-	g_return_val_if_fail (col != NULL, FALSE);
-
-	result = CLASS (model)->append_column (model, col);
-	if (result) {
-		gda_data_model_column_inserted (model,
-						gda_field_attributes_get_position (col));
+	g_return_val_if_fail (attrs != NULL, FALSE);
+	
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_append_column) 
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_append_column) (model, attrs);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
 	}
-
-	return result;
 }
 
 /**
@@ -768,26 +701,23 @@ gda_data_model_append_column (GdaDataModel *model, GdaFieldAttributes *col)
  * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
-gda_data_model_remove_column (GdaDataModel *model, const GdaFieldAttributes *col)
+gda_data_model_remove_column (GdaDataModel *model, gint col)
 {
-	gboolean result;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (CLASS (model)->remove_column != NULL, FALSE);
-	g_return_val_if_fail (col != NULL, FALSE);
 
-	result = CLASS (model)->remove_column (model, col);
-	if (result) {
-		gda_data_model_column_removed (model, gda_field_attributes_get_position (col));
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_remove_column) 
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_remove_column) (model, col);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
 	}
-
-	return result;
 }
 
 /**
  * gda_data_model_update_column
  * @model: a #GdaDataModel object.
  * @col: the column to be updated.
+ * @attrs: attributes for the column.
  *
  * Updates a column in the given data model. This results in the underlying
  * database row's values being changed.
@@ -795,19 +725,17 @@ gda_data_model_remove_column (GdaDataModel *model, const GdaFieldAttributes *col
  * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
-gda_data_model_update_column (GdaDataModel *model, const GdaFieldAttributes *col)
+gda_data_model_update_column (GdaDataModel *model, gint col, const GdaDataModelColumnAttributes *attrs)
 {
-	gboolean result;
-
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (CLASS (model)->update_column != NULL, FALSE);
-	g_return_val_if_fail (col != NULL, FALSE);
+	g_return_val_if_fail (attrs != NULL, FALSE);
 
-	result = CLASS (model)->update_column (model, col);
-	if (result) {
-		gda_data_model_column_updated (model, gda_field_attributes_get_position (col));
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_update_column) 
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_update_column) (model, col, attrs);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
 	}
-	return result;
 }
 
 /**
@@ -826,8 +754,8 @@ gda_data_model_update_column (GdaDataModel *model, const GdaFieldAttributes *col
  *
  * where "row" would be the row being read, and "user_data" the parameter
  * specified in @user_data in the call to gda_data_model_foreach.
- * This callback function can return %FALSE to stop the processing. If it
- * returns %TRUE, processing will continue until no rows remain.
+ * This callback function can return %FALSE to stop the processing. If
+ * it returns %TRUE, processing will continue until no rows remain.
  */
 void
 gda_data_model_foreach (GdaDataModel *model,
@@ -858,7 +786,7 @@ gda_data_model_foreach (GdaDataModel *model,
  * Checks whether this data model is in updating mode or not. Updating
  * mode is set to %TRUE when @gda_data_model_begin_update has been
  * called successfully, and is not set back to %FALSE until either
- * @gda_data_model_cancel_update or @gda_data_model_end_update have
+ * @gda_data_model_cancel_update or @gda_data_model_commit_update have
  * been called.
  *
  * Returns: %TRUE if updating mode, %FALSE otherwise.
@@ -867,7 +795,13 @@ gboolean
 gda_data_model_has_changed (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	return model->priv->updating;
+
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_has_changed) 
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_has_changed) (model);
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
+	}
 }
 
 /**
@@ -883,18 +817,19 @@ gboolean
 gda_data_model_begin_update (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (model->priv->updating == FALSE, FALSE);
 
 	if (!gda_data_model_is_updatable (model)) {
 		gda_log_error (_("Data model %p is not updatable"), model);
 		return FALSE;
 	}
 
-	model->priv->updating = TRUE;
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_begin_changes) 
+		(GDA_DATA_MODEL_GET_IFACE (model)->i_begin_changes) (model);
+	
 	g_signal_emit (G_OBJECT (model),
 		       gda_data_model_signals[BEGIN_UPDATE], 0);
-
-	return model->priv->updating;
+	
+	return TRUE;
 }
 
 /**
@@ -910,17 +845,23 @@ gboolean
 gda_data_model_cancel_update (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (model->priv->updating, FALSE);
 
-	g_signal_emit (G_OBJECT (model),
-		       gda_data_model_signals[CANCEL_UPDATE], 0);
-	model->priv->updating = FALSE;
-
-	return TRUE;
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_cancel_changes) {
+		gboolean status;
+		status = (GDA_DATA_MODEL_GET_IFACE (model)->i_cancel_changes) (model);
+		if (status)
+			g_signal_emit (G_OBJECT (model),
+				       gda_data_model_signals[CANCEL_UPDATE], 0);
+		return status;
+	}
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
+	}
 }
 
 /**
- * gda_data_model_end_update
+ * gda_data_model_commit_update
  * @model: a #GdaDataModel object.
  *
  * Approves all modifications and send them to the underlying
@@ -929,15 +870,22 @@ gda_data_model_cancel_update (GdaDataModel *model)
  * Returns: %TRUE on success, %FALSE if there was an error.
  */
 gboolean
-gda_data_model_end_update (GdaDataModel *model)
+gda_data_model_commit_update (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (model->priv->updating, FALSE);
 
-	g_signal_emit (G_OBJECT (model), gda_data_model_signals[END_UPDATE], 0);
-	model->priv->updating = FALSE;
-
-	return TRUE;
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_commit_changes) {
+		gboolean status;
+		status = (GDA_DATA_MODEL_GET_IFACE (model)->i_commit_changes) (model);
+		if (status)
+			g_signal_emit (G_OBJECT (model),
+				       gda_data_model_signals[COMMIT_UPDATE], 0);
+		return status;
+	}
+	else {
+		g_warning ("%s() method not supported\n", __FUNCTION__);
+		return FALSE;
+	}
 }
 
 static gchar *
@@ -958,10 +906,10 @@ export_to_separated (GdaDataModel *model, gchar sep)
 			str = g_string_append_c (str, '\n');
 
 		for (c = 0; c < cols; c++) {
-			const GdaValue *value;
+			GdaValue *value;
 			gchar *txt;
 
-			value = gda_data_model_get_value_at (model, c, r);
+			value = (GdaValue *) gda_data_model_get_value_at (model, c, r);
 			if (gda_value_get_type (value) == GDA_VALUE_TYPE_BOOLEAN)
 				txt = g_strdup (gda_value_get_boolean (value) ? "TRUE" : "FALSE");
 			else
@@ -1103,7 +1051,7 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 	rows = gda_data_model_get_n_rows (model);
 	cols = gda_data_model_get_n_columns (model);
 	for (i = 0; i < cols; i++) {
-		GdaFieldAttributes *fa;
+		GdaDataModelColumnAttributes *fa;
 		xmlNodePtr field;
 
 		fa = gda_data_model_describe_column (model, i);
@@ -1113,16 +1061,16 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 		}
 
 		field = xmlNewChild (node, NULL, "field", NULL);
-		xmlSetProp (field, "name", gda_field_attributes_get_name (fa));
-		xmlSetProp (field, "caption", gda_field_attributes_get_caption (fa));
-		xmlSetProp (field, "gdatype", gda_type_to_string (gda_field_attributes_get_gdatype (fa)));
-		xml_set_int (field, "size", gda_field_attributes_get_defined_size (fa));
-		xml_set_int (field, "scale", gda_field_attributes_get_scale (fa));
-		xml_set_boolean (field, "pkey", gda_field_attributes_get_primary_key (fa));
-		xml_set_boolean (field, "unique", gda_field_attributes_get_unique_key (fa));
-		xml_set_boolean (field, "isnull", gda_field_attributes_get_allow_null (fa));
-		xml_set_boolean (field, "auto_increment", gda_field_attributes_get_auto_increment (fa));
-		xmlSetProp (field, "references", gda_field_attributes_get_references (fa));
+		xmlSetProp (field, "name", gda_data_model_column_attributes_get_name (fa));
+		xmlSetProp (field, "caption", gda_data_model_column_attributes_get_caption (fa));
+		xmlSetProp (field, "gdatype", gda_type_to_string (gda_data_model_column_attributes_get_gdatype (fa)));
+		xml_set_int (field, "size", gda_data_model_column_attributes_get_defined_size (fa));
+		xml_set_int (field, "scale", gda_data_model_column_attributes_get_scale (fa));
+		xml_set_boolean (field, "pkey", gda_data_model_column_attributes_get_primary_key (fa));
+		xml_set_boolean (field, "unique", gda_data_model_column_attributes_get_unique_key (fa));
+		xml_set_boolean (field, "isnull", gda_data_model_column_attributes_get_allow_null (fa));
+		xml_set_boolean (field, "auto_increment", gda_data_model_column_attributes_get_auto_increment (fa));
+		xmlSetProp (field, "references", gda_data_model_column_attributes_get_references (fa));
 		xml_set_int (field, "position", i);
 	}
 	
@@ -1136,10 +1084,10 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
 			row = xmlNewChild (data, NULL, "row", NULL);
 			xml_set_int (row, "position", r);
 			for (c = 0; c < cols; c++) {
-				const GdaValue *value;
+				GdaValue *value;
 				gchar *str;
 
-				value = gda_data_model_get_value_at (model, c, r);
+				value = (GdaValue *) gda_data_model_get_value_at (model, c, r);
 				if (gda_value_get_type (value) == GDA_VALUE_TYPE_BOOLEAN)
 					str = g_strdup (gda_value_get_boolean (value) ? "TRUE" : "FALSE");
 				else
@@ -1212,7 +1160,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row)
 		}
 
 		if (retval)
-			gda_data_model_append_row (model, value_list);
+			gda_data_model_append_values (model, value_list);
 
 		g_list_free (value_list);
 	}
@@ -1226,7 +1174,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row)
 /**
  * gda_data_model_add_data_from_xml_node
  * @model: a #GdaDataModel.
- * @node: a XML node representing a <data> XML node.
+ * @node: a XML node representing a &lt;data&gt; XML node.
  *
  * Adds the data from a XML node to the given data model.
  *
@@ -1265,7 +1213,11 @@ const gchar *
 gda_data_model_get_command_text (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-	return model->priv->cmd_text;
+
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_command)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_get_command) (model, NULL);
+	else
+		return NULL;
 }
 
 /**
@@ -1274,16 +1226,19 @@ gda_data_model_get_command_text (GdaDataModel *model)
  * @txt: the command text.
  *
  * Sets the command text of the given @model.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
  */
-void
+gboolean
 gda_data_model_set_command_text (GdaDataModel *model, const gchar *txt)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	g_return_if_fail (txt != NULL);
-
-	if (model->priv->cmd_text)
-		g_free (model->priv->cmd_text);
-	model->priv->cmd_text = g_strdup (txt);
+	
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command) (model, txt, -1);
+	else
+		return FALSE;
 }
 
 /**
@@ -1298,7 +1253,14 @@ GdaCommandType
 gda_data_model_get_command_type (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), GDA_COMMAND_TYPE_INVALID);
-	return model->priv->cmd_type;
+
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_get_command) {
+		GdaCommandType type;
+		(GDA_DATA_MODEL_GET_IFACE (model)->i_get_command) (model, &type);
+		return type;
+	}
+	else
+		return GDA_COMMAND_TYPE_INVALID;
 }
 
 /**
@@ -1307,11 +1269,16 @@ gda_data_model_get_command_type (GdaDataModel *model)
  * @type: the type of the command (one of #GdaCommandType)
  *
  * Sets the type of command that generated this data model.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
  */
-void
+gboolean
 gda_data_model_set_command_type (GdaDataModel *model, GdaCommandType type)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-	model->priv->cmd_type = type;
+	if (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command)
+		return (GDA_DATA_MODEL_GET_IFACE (model)->i_set_command) (model, NULL, type);
+	else
+		return FALSE;
 }
 
