@@ -169,7 +169,7 @@ gda_oracle_provider_new (void)
 static const gchar *
 gda_oracle_provider_get_version (GdaServerProvider *provider)
 {
-	return VERSION;
+	return PACKAGE_VERSION;
 }
 
 /* open_connection handler for the GdaOracleProvider class */
@@ -201,7 +201,7 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 
         g_assert (priv_data != NULL);
         /* initialize Oracle */
-	result = OCIInitialize ((ub4) OCI_DEFAULT,
+	result = OCIInitialize ((ub4) OCI_THREADED,
 				(dvoid *) 0,
 				(dvoid * (*)(dvoid *, size_t)) 0,
 				(dvoid * (*)(dvoid *, dvoid *, size_t)) 0,
@@ -1120,7 +1120,7 @@ aggregate_foreach (gpointer key, gpointer value, gpointer data)
 	list = g_list_append(list, gda_value_new_string("NUMBER"));
 	list = g_list_append(list, gda_value_new_string(ptr->args));
 	list = g_list_append(list, gda_value_new_string(""));
-	gda_data_model_append_row (GDA_DATA_MODEL(data), list);
+	gda_data_model_append_values (GDA_DATA_MODEL(data), list);
 	for (listp = list; listp; listp = listp->next)
 		gda_value_free(listp->data);
 	g_list_free(list);
@@ -1332,7 +1332,10 @@ get_oracle_index_data (GdaConnection *cnc, const gchar *owner, const gchar *tbln
 	nrows = gda_data_model_get_n_rows (recset);
 
 	if (nrows == 0)
+	{
+		g_object_unref(recset);		
 		return NULL;
+	}
 
 	index_data = g_new0 (GdaOracleIndexData, 1);
 	references = g_string_new ("");
@@ -1391,7 +1394,7 @@ get_oracle_index_data (GdaConnection *cnc, const gchar *owner, const gchar *tbln
 			colname = g_string_new (newcolname->str);
 		}
 	}
-	g_free (recset);
+	g_object_unref(recset);
 	return h_table;
 }
 
@@ -1856,7 +1859,7 @@ get_tables_foreach(gpointer key, gpointer value, gpointer data)
     cols = g_list_append(cols, gda_value_new_string((const gchar *)value));
     cols = g_list_append(cols, gda_value_new_string(""));
     cols = g_list_append(cols, gda_value_new_string(""));
-    gda_data_model_append_row((GdaDataModel *)data, cols);
+    gda_data_model_append_values((GdaDataModel *)data, cols);
     return FALSE;
 }
 
@@ -2101,7 +2104,7 @@ get_oracle_types (GdaConnection *cnc, GdaParameterList *params)
         value_list = g_list_append (value_list, gda_value_new_string ("NULL"));
         value_list = g_list_append (value_list, gda_value_new_type (otp->type));
 
-	gda_data_model_append_row (GDA_DATA_MODEL (recset), value_list);
+	gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
         g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
         g_list_free (value_list);
     }
@@ -2114,7 +2117,7 @@ add_g_list_row (gpointer data, gpointer user_data)
 	GList *rowlist = data;
 	GdaDataModelArray *recset = user_data;
 
-	gda_data_model_append_row (GDA_DATA_MODEL (recset), rowlist);
+	gda_data_model_append_values (GDA_DATA_MODEL (recset), rowlist);
 	g_list_foreach (rowlist, (GFunc) gda_value_free, NULL);
 	g_list_free (rowlist);
 }
