@@ -164,7 +164,7 @@ gda_xml_provider_get_version (GdaServerProvider *provider)
 	GdaXmlProvider *dfprv = (GdaXmlProvider *) provider;
 
 	g_return_val_if_fail (GDA_IS_XML_PROVIDER (dfprv), NULL);
-	return VERSION;
+	return PACKAGE_VERSION;
 }
 
 /* open_connection handler for the GdaXmlProvider class */
@@ -528,7 +528,7 @@ get_table_fields (GdaConnection *cnc, GdaXmlDatabase *xmldb, GdaParameterList *p
 
 	cols = gda_data_model_get_n_columns (GDA_DATA_MODEL (table));
 	for (i = 0; i < cols; i++) {
-		GdaFieldAttributes *fa;
+		GdaDataModelColumnAttributes *fa;
 		GList *value_list = NULL;
 
 		fa = gda_data_model_describe_column (GDA_DATA_MODEL (table), i);
@@ -540,16 +540,16 @@ get_table_fields (GdaConnection *cnc, GdaXmlDatabase *xmldb, GdaParameterList *p
 			return NULL;
 		}
 
-		value_list = g_list_append (value_list, gda_value_new_string (gda_field_attributes_get_name (fa)));
-		value_list = g_list_append (value_list, gda_value_new_string (gda_type_to_string (gda_field_attributes_get_gdatype (fa))));
-		value_list = g_list_append (value_list, gda_value_new_integer (gda_field_attributes_get_defined_size (fa)));
-		value_list = g_list_append (value_list, gda_value_new_integer (gda_field_attributes_get_scale (fa)));
+		value_list = g_list_append (value_list, gda_value_new_string (gda_data_model_column_attributes_get_name (fa)));
+		value_list = g_list_append (value_list, gda_value_new_string (gda_type_to_string (gda_data_model_column_attributes_get_gdatype (fa))));
+		value_list = g_list_append (value_list, gda_value_new_integer (gda_data_model_column_attributes_get_defined_size (fa)));
+		value_list = g_list_append (value_list, gda_value_new_integer (gda_data_model_column_attributes_get_scale (fa)));
 		value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
 		value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
 		value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
 		value_list = g_list_append (value_list, gda_value_new_string (""));
 
-		gda_field_attributes_free (fa);
+		gda_data_model_column_attributes_free (fa);
 	}
 
 	return GDA_DATA_MODEL (recset);
@@ -569,7 +569,7 @@ add_string_row (GdaDataModelArray *recset, const gchar *str)
 	list.next = NULL;
 	list.prev = NULL;
 
-	gda_data_model_append_row (GDA_DATA_MODEL (recset), &list);
+	gda_data_model_append_values (GDA_DATA_MODEL (recset), &list);
 
 	gda_value_free (value);
 }
@@ -612,7 +612,7 @@ get_tables (GdaConnection *cnc, GdaXmlDatabase *xmldb)
 
 			value = gda_value_new_string ((const gchar *) l->data);
 			value_list = g_list_append (NULL, value);
-			gda_data_model_append_row (GDA_DATA_MODEL (recset),
+			gda_data_model_append_values (GDA_DATA_MODEL (recset),
 						   (const GList *) value_list);
 
 			gda_value_free (value);
@@ -625,6 +625,23 @@ get_tables (GdaConnection *cnc, GdaXmlDatabase *xmldb)
 	return GDA_DATA_MODEL (recset);
 }
 
+static void
+add_type_row (GdaDataModelArray *recset, const gchar *type, const gchar *owner,
+	      const gchar *comments, GdaValueType gda_type)
+{
+	GList *value_list = NULL;
+
+	value_list = g_list_append (value_list, gda_value_new_string (type));
+	value_list = g_list_append (value_list, gda_value_new_string (owner));
+	value_list = g_list_append (value_list, gda_value_new_string (comments));
+	value_list = g_list_append (value_list, gda_value_new_type (gda_type));
+
+	gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
+
+	g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+	g_list_free (value_list);
+}
+
 static GdaDataModel *
 get_types (GdaConnection *cnc)
 {
@@ -633,25 +650,34 @@ get_types (GdaConnection *cnc)
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
 	/* create the recordset */
-	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (1));
-	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Name"));
+	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (4));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("Owner"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
 
 	/* fill the recordset */
-	add_string_row (recset, "bigint");
-	add_string_row (recset, "binary");
-	add_string_row (recset, "boolean");
-	add_string_row (recset, "date");
-	add_string_row (recset, "double");
-	add_string_row (recset, "integer");
-	add_string_row (recset, "list");
-	add_string_row (recset, "numeric");
-	add_string_row (recset, "point");
-	add_string_row (recset, "single");
-	add_string_row (recset, "smallint");
-	add_string_row (recset, "string");
-	add_string_row (recset, "time");
-	add_string_row (recset, "timestamp");
-	add_string_row (recset, "tinyint");
+	add_type_row (recset, "bigint", "", _("Big integer"), GDA_VALUE_TYPE_BIGINT);
+	add_type_row (recset, "biguint", "", _("Big unsigned integer"), GDA_VALUE_TYPE_BIGUINT);
+	add_type_row (recset, "binary", "", _("Binary"), GDA_VALUE_TYPE_BINARY);
+	add_type_row (recset, "boolean", "", _("Boolean"), GDA_VALUE_TYPE_BOOLEAN);
+	add_type_row (recset, "date", "", _("Date"), GDA_VALUE_TYPE_DATE);
+	add_type_row (recset, "double", "", _("Double precision"), GDA_VALUE_TYPE_DOUBLE);
+	add_type_row (recset, "integer", "", _("Integer"), GDA_VALUE_TYPE_INTEGER);
+	add_type_row (recset, "list", "", _("List"), GDA_VALUE_TYPE_LIST);
+	add_type_row (recset, "money", "", _("Money"), GDA_VALUE_TYPE_MONEY);
+	add_type_row (recset, "numeric", "", _("Numeric"), GDA_VALUE_TYPE_NUMERIC);
+	add_type_row (recset, "point", "", _("Point"), GDA_VALUE_TYPE_GEOMETRIC_POINT);
+	add_type_row (recset, "single", "", _("Single precision"), GDA_VALUE_TYPE_SINGLE);
+	add_type_row (recset, "smallint", "", _("Small integer"), GDA_VALUE_TYPE_SMALLINT);
+	add_type_row (recset, "smalluint", "", _("Small unsigned integer"), GDA_VALUE_TYPE_SMALLUINT);
+	add_type_row (recset, "string", "", _("String"), GDA_VALUE_TYPE_STRING);
+	add_type_row (recset, "time", "", _("Time"), GDA_VALUE_TYPE_TIME);
+	add_type_row (recset, "timestamp", "", _("Timestamp"), GDA_VALUE_TYPE_TIMESTAMP);
+	add_type_row (recset, "tinyint", "", _("Tiny integer"), GDA_VALUE_TYPE_TINYINT);
+	add_type_row (recset, "tinyuint", "", _("Tiny unsigned integer"), GDA_VALUE_TYPE_TINYUINT);
+	add_type_row (recset, "type", "", _("Type"), GDA_VALUE_TYPE_TYPE);
+	add_type_row (recset, "uinteger", "", _("Unsigned integer"), GDA_VALUE_TYPE_UINTEGER);
 
 	return GDA_DATA_MODEL (recset);
 }
@@ -681,7 +707,7 @@ gda_xml_provider_get_schema (GdaServerProvider *provider,
 		return get_tables (cnc, xmldb);
 	case GDA_CONNECTION_SCHEMA_TYPES :
 		return get_types (cnc);
-	default :
+	default : break;
 	}
 
 	return NULL;
